// Copyright (c) 2015, Intel Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
// 3. Neither the name of the copyright holder nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package org.moe.samples.elements.ios;

import org.moe.natj.general.Pointer;
import org.moe.natj.general.ann.ByValue;
import org.moe.natj.general.ann.Mapped;
import org.moe.natj.general.ann.Owned;
import org.moe.natj.general.ann.RegisterOnStartup;
import org.moe.natj.general.ptr.VoidPtr;
import org.moe.natj.objc.ObjCRuntime;
import org.moe.natj.objc.ann.ObjCClassName;
import org.moe.natj.objc.ann.Selector;
import org.moe.natj.objc.map.ObjCObjectMapper;
import org.moe.samples.elements.common.AtomicElement;

import apple.coregraphics.c.CoreGraphics;
import apple.coregraphics.struct.CGRect;
import apple.coregraphics.struct.CGSize;
import apple.foundation.NSNumber;
import apple.foundation.NSString;
import apple.foundation.c.Foundation;
import apple.uikit.UIBarButtonItem;
import apple.uikit.UIButton;
import apple.uikit.UIImage;
import apple.uikit.UIImageView;
import apple.uikit.UIView;
import apple.uikit.UIViewController;
import apple.uikit.enums.UIControlEvents;
import apple.uikit.enums.UIControlState;
import apple.uikit.enums.UIViewAnimationTransition;

@org.moe.natj.general.ann.Runtime(ObjCRuntime.class)
@ObjCClassName("AtomicElementViewController")
@RegisterOnStartup
public class AtomicElementViewController extends UIViewController {

    @Owned
    @Selector("alloc")
    public static native AtomicElementViewController alloc();

    @Owned
    @Selector("init")
    public native AtomicElementViewController init();

    protected AtomicElementViewController(Pointer peer) {
        super(peer);
    }

    private static final float kFlipTransitionDuration = 0.75f;
    private static final float reflectionFraction = 0.35f;
    private static final float reflectionOpacity = 0.5f;

    private AtomicElement element = null;
    private boolean frontViewIsVisible;
    private AtomicElementView atomicElementView;
    private UIImageView reflectionView;
    private AtomicElementFlippedView atomicElementFlippedView;
    private UIButton flipIndicatorButton;

    @Override
    public void viewDidLoad() {
        frontViewIsVisible = true;

        CGSize preferredAtomicElementViewSize = AtomicElementView.preferredViewSize();

        CGSize viewSize = view().bounds().size();

        CGRect viewRect = CoreGraphics.CGRectMake((viewSize.width() - preferredAtomicElementViewSize.width()) / 2,
                (viewSize.height() - preferredAtomicElementViewSize.height()) / 2 - 40,
                preferredAtomicElementViewSize.width(),
                preferredAtomicElementViewSize.height()
        );

        // Create the atomic element view
        AtomicElementView localAtomicElementView = (AtomicElementView) AtomicElementView.alloc().initWithFrame(viewRect);
        atomicElementView = localAtomicElementView;

        // Add the atomic element view to the view controller's view
        atomicElementView.setElement(element);
        view().addSubview(atomicElementView);

        atomicElementView.setViewController(this); // TODO this could be a problem - check 'viewDidDisappear' for more information

        // Create the atomic element flipped view
        AtomicElementFlippedView localAtomicElementFlippedView = (AtomicElementFlippedView) AtomicElementFlippedView.alloc().initWithFrame(viewRect);
        atomicElementFlippedView = localAtomicElementFlippedView;

        atomicElementFlippedView.setElement(element);
        atomicElementFlippedView.setViewController(this); // TODO this could be a problem - check 'viewDidDisappear' for more information

        // Create the reflection view
//        CGRect reflectionRect = viewRect;
//
//        // The reflection is a fraction of the size of the view being reflected
//        reflectionRect.getSize().setHeight(reflectionRect.getSize().getHeight() * reflectionFraction);
//
//        // and is offset to be at the bottom of the view being reflected
//        reflectionRect.getOrigin().setY(viewRect.getSize().getHeight());
//
//        UIImageView localReflectionImageView = (UIImageView) UIImageView.alloc().init(reflectionRect);
//        this.reflectionView = localReflectionImageView;
//
//        // Determine the size of the reflection to create
//        int reflectionHeight = (int) (this.atomicElementView.getBounds().getSize().getHeight() * reflectionFraction);
//
//        // Create the reflection image, assign it to the UIImageView and add the image view to the view controller's view
//        // self.reflectionView.image = [self.atomicElementView reflectedImageRepresentationWithHeight:reflectionHeight];
//        this.reflectionView.setImage(this.atomicElementView.reflectedImageRepresentationWithHeight(reflectionHeight));
//        this.reflectionView.setAlpha(reflectionOpacity);

        //getView().addSubview(this.reflectionView);

        // Setup our flip indicator button (placed as a nav bar item to the right)
        UIButton localFlipIndicator = UIButton.alloc().initWithFrame(CoreGraphics.CGRectMake(0.0, 0.0, 30.0, 30.0));
        this.flipIndicatorButton = localFlipIndicator;

        // Front view is always visible at first
        UIImage flipper_list_blue = UIImage.imageNamed("flipper_list_blue.png");
        this.flipIndicatorButton.setBackgroundImageForState(flipper_list_blue, UIControlState.Normal);

        UIBarButtonItem flipButtonBarItem;
        flipButtonBarItem = UIBarButtonItem.alloc().initWithCustomView(this.flipIndicatorButton);
        flipIndicatorButton.addTargetActionForControlEvents(this, Foundation.NSSelectorFromString("flipCurrentView"), UIControlEvents.TouchDown);
        this.navigationItem().setRightBarButtonItemAnimated(flipButtonBarItem, true);
    }

    private CGRect getRectForSubViews(CGSize superviewSize) {
        CGSize preferredViewSize = AtomicElementView.preferredViewSize();

        CGRect viewRect = CoreGraphics.CGRectMake(
                (superviewSize.width() - preferredViewSize.width()) / 2,
                (superviewSize.height() - preferredViewSize.height()) / 2,
                preferredViewSize.width(),
                preferredViewSize.height()
        );

        return viewRect;
    }

    @Override
    public void viewWillTransitionToSizeWithTransitionCoordinator(@ByValue CGSize size, @Mapped(ObjCObjectMapper.class) Object coordinator) {
        CGRect viewRect = getRectForSubViews(size);
        atomicElementView.setFrame(viewRect);
        atomicElementFlippedView.setFrame(viewRect);
    }

    public void setElement(AtomicElement element) {
        this.element = element;
    }

    @Selector("flipCurrentView")
    public void flipCurrentView() {
//        NSUInteger reflectionHeight;
        UIImage reflectedImage;

        // Disable user interaction during the flip animation
        this.view().setUserInteractionEnabled(false);
        this.flipIndicatorButton.setUserInteractionEnabled(false);

        // Setup the animation group
        UIView.beginAnimationsContext(null, null);
        UIView.setAnimationDuration_static(kFlipTransitionDuration);
        UIView.setAnimationDelegate(this);
        UIView.setAnimationDidStopSelector(Foundation.NSSelectorFromString("myTransitionDidStop:finished:context:"));

        // Swap the views and transition
        if (frontViewIsVisible) {
            UIView.setAnimationTransitionForViewCache(UIViewAnimationTransition.FlipFromRight, this.view(), true);
            atomicElementView.removeFromSuperview();
            view().addSubview(atomicElementFlippedView);

            // Update the reflection image for the new view
        } else {
            UIView.setAnimationTransitionForViewCache(UIViewAnimationTransition.FlipFromLeft, this.view(), true);
            atomicElementFlippedView.removeFromSuperview();
            view().addSubview(atomicElementView);

            // Update the reflection image for the new view
        }
        UIView.commitAnimations();

        // Swap the nav bar button views
        UIView.beginAnimationsContext(null, null);
        UIView.setAnimationDuration_static(kFlipTransitionDuration);

        // TODO ATTENTION: check if 'UIView.setAnimationDelegate' it holds a strong reference to its delegate.
        UIView.setAnimationDelegate(this);

        if (frontViewIsVisible) {
            UIView.setAnimationTransitionForViewCache(UIViewAnimationTransition.FlipFromRight, flipIndicatorButton, true);
            UIImage flipperImage = PeriodicElements.flipperImageForAtomicElementNavigationItem(element);
            this.flipIndicatorButton.setBackgroundImageForState(flipperImage, UIControlState.Normal);
        } else {
            UIView.setAnimationTransitionForViewCache(UIViewAnimationTransition.FlipFromLeft, flipIndicatorButton, true);
            UIImage flipper_list_blue = UIImage.imageNamed("flipper_list_blue.png");
            this.flipIndicatorButton.setBackgroundImageForState(flipper_list_blue, UIControlState.Normal);
        }

        UIView.commitAnimations();

        // Invert the front view state
        frontViewIsVisible = !frontViewIsVisible;
    }



    //
    // TODO verify this
    // Removing "viewDidDisappear" and "viewWillAppear" causes a bigger memory leak.
    // in ObjC: Holding a strong reference to the view controller (as shown here) causes a retain cycle.
    // in Java: With garbage collection retain cycles shouldn't happen.
    // The main question here is: What memory management will be used in this case?
    // Looking on the memory usage (while 'profiling') this code is needed to avoid a memory leak!
    // To verify this behavior comment 'viewDidDisappear' and 'viewWillAppear' out.
    //
    // PS: even with these two methods it will leak a little bit and the memory usage will slowly
    // increase over time while pushing and popping this controller (~ 1 min = + 5MB).
    //
    @Override
    public void viewDidDisappear(boolean animated) {
        super.viewDidDisappear(animated);

        if (this.atomicElementFlippedView!=null) {
            this.atomicElementFlippedView.viewController=null;
        }

        if (this.atomicElementView!=null) {
            this.atomicElementView.viewController=null;
        }

        // Remove any other dependencies - just to be sure.
        // After some tests:
        // The following code is not really needed, but
        // it also reduces the memory usage after viewDidDisappear noticeable.
        /*
        if (this.atomicElementFlippedView!=null && this.atomicElementFlippedView.superview()!=null) {
            this.atomicElementFlippedView.removeFromSuperview();
        }
        if (this.atomicElementView!=null && this.atomicElementView.superview()!=null) {
            this.atomicElementView.removeFromSuperview();
        }

        if (this.navigationItem()!=null){
            this.navigationItem().setRightBarButtonItem(null);
            flipIndicatorButton.removeTargetActionForControlEvents(this, Foundation.NSSelectorFromString("flipCurrentView"), UIControlEvents.TouchDown);
        }
        */
    }

    @Override
    public void viewWillAppear(boolean animated) {
        super.viewWillAppear(animated);

        if (this.atomicElementFlippedView!=null) {
            this.atomicElementFlippedView.viewController = this;
        }
        if (this.atomicElementView!=null) {
            this.atomicElementView.viewController = this;
        }

        // If the views are not in the view hierarchy add them back.
        // This is just needed if the views were removed in viewDidDisappear.
        if (frontViewIsVisible) {
            if (this.atomicElementView!=null && this.atomicElementView.superview()==null) {
                view().addSubview(atomicElementView);
            }
        } else {
            if (this.atomicElementFlippedView!=null && this.atomicElementFlippedView.superview()==null) {
                view().addSubview(atomicElementFlippedView);
            }
        }
        if (this.navigationItem()!=null && this.navigationItem().rightBarButtonItem()==null){
            UIBarButtonItem flipButtonBarItem;
            flipButtonBarItem = UIBarButtonItem.alloc().initWithCustomView(this.flipIndicatorButton);
            flipIndicatorButton.addTargetActionForControlEvents(this, Foundation.NSSelectorFromString("flipCurrentView"), UIControlEvents.TouchDown);
            this.navigationItem().setRightBarButtonItemAnimated(flipButtonBarItem, false);
        }
    }

    @Selector("myTransitionDidStop:finished:context:")
    public void myTransitionDidStop(NSString animationID, NSNumber finished, VoidPtr context) {
        // Re-enable user interaction when the flip animation is completed
        this.view().setUserInteractionEnabled(true);
        this.flipIndicatorButton.setUserInteractionEnabled(true);
    }
}
